from .widgets.timeserieswidget import *
from .widgets.unitwaveformswidget import *
from .widgets.correlogramswidget import *
from .widgets.sortingperformancewidget import *
from .widgets.electrodegeometrywidget import *
from .widgets.confusionmatrixwidget import *
from .widgets.rasterswidget import *
from .widgets.isidistwidget import *
from .widgets.multicompgraphwidget import *
from .widgets.amplitudewidget import *
from .widgets.featurewidgets import *
from .widgets.spectrumwidget import *
from .widgets.agreementmatrixwidget import *

from .version import version as __version__
